Werkzeug Documentation

# Werkzeug

# Utilities

You can import all these objects directly from `werkzeug`.

## URL Helpers

class `werkzeug.` `Href`(base='./', charset='utf-8', sort=False, key=None)

  Implements a callable that constructs URLs with the given base. The function can be called with any number of positional and keyword arguments which than are used to assemble the URL. Works with URLs and posix paths.

  Positional arguments are appended as individual segments to the path of the URL:

      >>> href = Href('/foo')
      >>> href('bar', 23)
      '/foo/bar/23'
      >>> href('foo', bar=23)
      '/foo/foo?bar=23'

  If any of the arguments (positional or keyword) evaluates to None it will be skipped. If no keyword arguments are given the last argument can be a `dict` or `MultiDict` (or any other dict subclass), otherwise the keyword arguments are used for the query parameters, cutting off the first trailing underscore of the parameter name:

      >>> href(is_=42)
      '/foo?is=42'
      >>> href({'foo': 'bar'})
      '/foo?foo=bar'

  Combining of both methods is not allowed:

      >>> href({'foo': 'bar'}, bar=42)
      Traceback (most recent call last):
        ...
      TypeError: keyword arguments and query-dicts can't be combined

  Accessing attributes on the href object creates a new href object with the attribute name as prefix:

      >>> bar_href = href.bar
      >>> bar_href("blub")
      '/foo/bar/blub'

  If sort is set to True the items are sorted by key or the default sorting algorithm:

      >>> href = Href("/", sort=True)
      >>> href(a=1, b=2, c=3)
      '/?a=1&b=2&c=3'

  New in version 0.5: sort and key were added.

`werkzeug.` `url_decode`(s, charset='utf-8', decode_keys=False, include_empty=True, errors='ignore', separator='&', cls=None)

  Parse a querystring and return it as `MultiDict`. Per default only values are decoded into unicode strings. If decode_keys is set to True the same will happen for keys.

  Per default a missing value for a key will default to an empty key. If you don’t want that behavior you can set include_empty to False.

  Per default encoding errors are ignored. If you want a different behavior you can set errors to `'replace'` or `'strict'`. In strict mode a HTTPUnicodeError is raised.

  Changed in version 0.5: In previous versions “;” and “&” could be used for url decoding. This changed in 0.5 where only “&” is supported. If you want to use “;” instead a different separator can be provided.

  The cls parameter was added.

  Parameters: | s – a string with the query string to decode. charset – the charset of the query string. decode_keys – set to True if you want the keys to be decoded as well. include_empty – Set to False if you don’t want empty values to appear in the dict. errors – the decoding error behavior. separator – the pair separator to be used, defaults to `&` cls – an optional dict class to use. If this is not specified or None the default `MultiDict` is used.

`werkzeug.` `url_encode`(obj, charset='utf-8', encode_keys=False, sort=False, key=None, separator='&')

  URL encode a dict/MultiDict. If a value is None it will not appear in the result string. Per default only values are encoded into the target charset strings. If encode_keys is set to `True` unicode keys are supported too.

  If sort is set to True the items are sorted by key or the default sorting algorithm.

  New in version 0.5: sort, key, and separator were added.

  Parameters: | obj – the object to encode into a query string. charset – the charset of the query string. encode_keys – set to True if you have unicode keys. sort – set to True if you want parameters to be sorted by key. separator – the separator to be used for the pairs. key – an optional function to be used for sorting. For more details check out the `sorted()` documentation.

`werkzeug.` `url_quote`(s, charset='utf-8', safe='/:')

  URL encode a single string with a given encoding.

  Parameters: | s – the string to quote. charset – the charset to be used. safe – an optional sequence of safe characters.

`werkzeug.` `url_quote_plus`(s, charset='utf-8', safe='')

  URL encode a single string with the given encoding and convert whitespace to “+”.

  Parameters: | s – the string to quote. charset – the charset to be used. safe – an optional sequence of safe characters.

`werkzeug.` `url_unquote`(s, charset='utf-8', errors='ignore')

  URL decode a single string with a given decoding.

  Per default encoding errors are ignored. If you want a different behavior you can set errors to `'replace'` or `'strict'`. In strict mode a HTTPUnicodeError is raised.

  Parameters: | s – the string to unquote. charset – the charset to be used. errors – the error handling for the charset decoding.

`werkzeug.` `url_unquote_plus`(s, charset='utf-8', errors='ignore')

  URL decode a single string with the given decoding and decode a “+” to whitespace.

  Per default encoding errors are ignored. If you want a different behavior you can set errors to `'replace'` or `'strict'`. In strict mode a HTTPUnicodeError is raised.

  Parameters: | s – the string to unquote. charset – the charset to be used. errors – the error handling for the charset decoding.

`werkzeug.` `url_fix`(s, charset='utf-8')

  Sometimes you get an URL by a user that just isn’t a real URL because it contains unsafe characters like ‘ ‘ and so on. This function can fix some of the problems in a similar way browsers handle data entered by the user:

      >>> url_fix(u'http://de.wikipedia.org/wiki/Elf (Begriffskl\xe4rung)')
      'http://de.wikipedia.org/wiki/Elf%20%28Begriffskl%C3%A4rung%29'

  Parameters: | s – the string with the URL to fix. charset – The target charset for the URL if the url was given as unicode string.

`werkzeug.` `uri_to_iri`(uri, charset='utf-8', errors='ignore')

  Converts a URI in a given charset to a IRI.

  Examples for URI versus IRI

      >>> uri_to_iri('http://xn--n3h.net/')
      u'http://\u2603.net/'
      >>> uri_to_iri('http://%C3%BCser:p%C3%A4ssword@xn--n3h.net/p%C3%A5th')
      u'http://\xfcser:p\xe4ssword@\u2603.net/p\xe5th'

  Query strings are left unchanged:

      >>> uri_to_iri('/?foo=24&x=%26%2f')
      u'/?foo=24&x=%26%2f'

  New in version 0.6.

  Parameters: | uri – the URI to convert charset – the charset of the URI errors – the error handling on decode

`werkzeug.` `iri_to_uri`(iri, charset='utf-8')

  Converts any unicode based IRI to an acceptable ASCII URI. Werkzeug always uses utf-8 URLs internally because this is what browsers and HTTP do as well. In some places where it accepts an URL it also accepts a unicode IRI and converts it into a URI.

  Examples for IRI versus URI:

      >>> iri_to_uri(u'http://☃.net/')
      'http://xn--n3h.net/'
      >>> iri_to_uri(u'http://üser:pässword@☃.net/påth')
      'http://%C3%BCser:p%C3%A4ssword@xn--n3h.net/p%C3%A5th'

  New in version 0.6.

  Parameters: | iri – the iri to convert charset – the charset for the URI

`werkzeug.` `extract_path_info`(environ_or_baseurl, path_or_url, charset='utf-8', errors='ignore', collapse_http_schemes=True)

  Extracts the path info from the given URL (or WSGI environment) and path. The path info returned is a unicode string, not a bytestring suitable for a WSGI environment. The URLs might also be IRIs.

  If the path info could not be determined, None is returned.

  Some examples:

      >>> extract_path_info('http://example.com/app', '/app/hello')
      u'/hello'
      >>> extract_path_info('http://example.com/app',
      ...                   'https://example.com/app/hello')
      u'/hello'
      >>> extract_path_info('http://example.com/app',
      ...                   'https://example.com/app/hello',
      ...                   collapse_http_schemes=False) is None
      True

  Instead of providing a base URL you can also pass a WSGI environment.

  New in version 0.6.

  Parameters: | environ_or_baseurl – a WSGI environment dict, a base URL or base IRI. This is the root of the application. path_or_url – an absolute path from the server root, a relative path (in which case it’s the path info) or a full URL. Also accepts IRIs and unicode parameters. charset – the charset for byte data in URLs errors – the error handling on decode collapse_http_schemes – if set to False the algorithm does not assume that http and https on the same server point to the same resource.

## HTML Helpers

class `werkzeug.` `HTMLBuilder`(dialect)

  Helper object for HTML generation.

  Per default there are two instances of that class. The html one, and the xhtml one for those two dialects. The class uses keyword parameters and positional parameters to generate small snippets of HTML.

  Keyword parameters are converted to XML/SGML attributes, positional arguments are used as children. Because Python accepts positional arguments before keyword arguments it’s a good idea to use a list with the star-syntax for some children:

      >>> html.p(class_='foo', *[html.a('foo', href='foo.html'), ' ',
      ...                        html.a('bar', href='bar.html')])
      u'<p class="foo"><a href="foo.html">foo</a> <a href="bar.html">bar</a></p>'

  This class works around some browser limitations and can not be used for arbitrary SGML/XML generation. For that purpose lxml and similar libraries exist.

  Calling the builder escapes the string passed:

      >>> html.p(html("<foo>"))
      u'<p>&lt;foo&gt;</p>'

`werkzeug.` `escape`(s, quote=False)

  Replace special characters “&”, “<” and “>” to HTML-safe sequences. If the optional flag quote is True, the quotation mark character (“) is also translated.

  There is a special handling for None which escapes to an empty string.

  Parameters: | s – the string to escape. quote – set to true to also escape double quotes.

`werkzeug.` `unescape`(s)

  The reverse function of escape. This unescapes all the HTML entities, not only the XML entities inserted by escape.

  Parameter: | s – the string to unescape.

## General Helpers

class `werkzeug.` `UserAgent`(environ_or_string)

  Represents a user agent. Pass it a WSGI environment or a user agent string and you can inspect some of the details from the user agent string via the attributes. The following attributes exist:

  `string`
    the raw user agent string
  `platform`

    the browser platform. The following platforms are currently recognized:

    - aix
    - amiga
    - android
    - bsd
    - hpux
    - iphone
    - irix
    - linux
    - macos
    - sco
    - solaris
    - wii
    - windows
  `browser`

    the name of the browser. The following browsers are currently recognized:

    - aol *
    - ask *
    - camino
    - chrome
    - firefox
    - galeon
    - google *
    - kmeleon
    - konqueror
    - links
    - lynx
    - msie
    - msn
    - netscape
    - opera
    - safari
    - seamonkey
    - webkit
    - yahoo *

    (Browsers maked with a star (`*`) are crawlers.)

  `version`
    the version of the browser
  `language`
    the language of the browser

class `werkzeug.` `cached_property`(func, name=None, doc=None, writeable=False)

  A decorator that converts a function into a lazy property. The function wrapped is called the first time to retrieve the result and then that calculated result is used the next time you access the value:

      class Foo(object):

          @cached_property
          def foo(self):
              # calculate something important here
              return 42

  The class has to have a __dict__ in order for this property to work.

  Changed in version 0.6: the writeable attribute and parameter was deprecated. If a cached property is writeable or not has to be documented now. For performance reasons the implementation does not honor the writeable setting and will always make the property writeable.

class `werkzeug.` `environ_property`(name, default=None, load_func=None, dump_func=None, read_only=None, doc=None)

  Maps request attributes to environment variables. This works not only for the Werzeug request object, but also any other class with an environ attribute:

      >>> class Test(object):
      ...     environ = {'key': 'value'}
      ...     test = environ_property('key')
      >>> var = Test()
      >>> var.test
      'value'

  If you pass it a second value it’s used as default if the key does not exist, the third one can be a converter that takes a value and converts it. If it raises `ValueError` or `TypeError` the default value is used. If no default value is provided None is used.

  Per default the property is read only. You have to explicitly enable it by passing `read_only=False` to the constructor.

class `werkzeug.` `header_property`(name, default=None, load_func=None, dump_func=None, read_only=None, doc=None)
  Like environ_property but for headers.

`werkzeug.` `parse_cookie`(header, charset='utf-8', errors='ignore', cls=None)

  Parse a cookie. Either from a string or WSGI environ.

  Per default encoding errors are ignored. If you want a different behavior you can set errors to `'replace'` or `'strict'`. In strict mode a `HTTPUnicodeError` is raised.

  Changed in version 0.5: This function now returns a `TypeConversionDict` instead of a regular dict. The cls parameter was added.

  Parameters: | header – the header to be used to parse the cookie. Alternatively this can be a WSGI environment. charset – the charset for the cookie values. errors – the error behavior for the charset decoding. cls – an optional dict class to use. If this is not specified or None the default `TypeConversionDict` is used.

`werkzeug.` `dump_cookie`(key, value='', max_age=None, expires=None, path='/', domain=None, secure=None, httponly=False, charset='utf-8', sync_expires=True)

  Creates a new Set-Cookie header without the `Set-Cookie` prefix The parameters are the same as in the cookie Morsel object in the Python standard library but it accepts unicode data, too.

  Parameters: | max_age – should be a number of seconds, or None (default) if the cookie should last only as long as the client’s browser session. Additionally timedelta objects are accepted, too. expires – should be a datetime object or unix timestamp. path – limits the cookie to a given path, per default it will span the whole domain. domain – Use this if you want to set a cross-domain cookie. For example, `domain=".example.com"` will set a cookie that is readable by the domain `www.example.com`, `foo.example.com` etc. Otherwise, a cookie will only be readable by the domain that set it. secure – The cookie will only be available via HTTPS httponly – disallow JavaScript to access the cookie. This is an extension to the cookie standard and probably not supported by all browsers. charset – the encoding for unicode values. sync_expires – automatically set expires if max_age is defined but expires not.

`werkzeug.` `redirect`(location, code=302)

  Return a response object (a WSGI application) that, if called, redirects the client to the target location. Supported codes are 301, 302, 303, 305, and 307. 300 is not supported because it’s not a real redirect and 304 because it’s the answer for a request with a request with defined If-Modified-Since headers.

  New in version 0.6: The location can now be a unicode string that is encoded using the `iri_to_uri()` function.

  Parameters: | location – the location the response should redirect to. code – the redirect status code.

`werkzeug.` `append_slash_redirect`(environ, code=301)

  Redirect to the same URL but with a slash appended. The behavior of this function is undefined if the path ends with a slash already.

  Parameters: | environ – the WSGI environment for the request that triggers the redirect. code – the status code for the redirect.

`werkzeug.` `import_string`(import_name, silent=False)

  Imports an object based on a string. This is useful if you want to use import paths as endpoints or something similar. An import path can be specified either in dotted notation (`xml.sax.saxutils.escape`) or with a colon as object delimiter (`xml.sax.saxutils:escape`).

  If silent is True the return value will be None if the import fails.

  Parameters: | import_name – the dotted name for the object to import. silent – if set to True import errors are ignored and None is returned instead.
  Returns: | imported object

`werkzeug.` `find_modules`(import_path, include_packages=False, recursive=False)

  Find all the modules below a package. This can be useful to automatically import all views / controllers so that their metaclasses / function decorators have a chance to register themselves on the application.

  Packages are not returned unless include_packages is True. This can also recursively list modules but in that case it will import all the packages to get the correct load path of that module.

  Parameters: | import_name – the dotted name for the package to find child modules. include_packages – set to True if packages should be returned, too. recursive – set to True if recursion should happen.
  Returns: | generator

`werkzeug.` `validate_arguments`(func, args, kwargs, drop_extra=True)

  Check if the function accepts the arguments and keyword arguments. Returns a new `(args, kwargs)` tuple that can safely be passed to the function without causing a TypeError because the function signature is incompatible. If drop_extra is set to True (which is the default) any extra positional or keyword arguments are dropped automatically.

  The exception raised provides three attributes:

  missing
    A set of argument names that the function expected but where missing.
  extra
    A dict of keyword arguments that the function can not handle but where provided.
  extra_positional
    A list of values that where given by positional argument but the function cannot accept.

  This can be useful for decorators that forward user submitted data to a view function:

      from werkzeug import ArgumentValidationError, validate_arguments

      def sanitize(f):
          def proxy(request):
              data = request.values.to_dict()
              try:
                  args, kwargs = validate_arguments(f, (request,), data)
              except ArgumentValidationError:
                  raise BadRequest('The browser failed to transmit all '
                                   'the data expected.')
              return f(*args, **kwargs)
          return proxy

  Parameters: | func – the function the validation is performed against. args – a tuple of positional arguments. kwargs – a dict of keyword arguments. drop_extra – set to False if you don’t want extra arguments to be silently dropped.
  Returns: | tuple in the form `(args, kwargs)`.

`werkzeug.` `secure_filename`(filename)

  Pass it a filename and it will return a secure version of it. This filename can then safely be stored on a regular file system and passed to `os.path.join()`. The filename returned is an ASCII only string for maximum portability.

  On windows system the function also makes sure that the file is not named after one of the special device files.

      >>> secure_filename("My cool movie.mov")
      'My_cool_movie.mov'
      >>> secure_filename("../../../etc/passwd")
      'etc_passwd'
      >>> secure_filename(u'i contain cool \xfcml\xe4uts.txt')
      'i_contain_cool_umlauts.txt'

  The function might return an empty filename. It’s your responsibility to ensure that the filename is unique and that you generate random filename if the function returned an empty one.

  New in version 0.5.

  Parameter: | filename – the filename to secure

`werkzeug.` `bind_arguments`(func, args, kwargs)

  Bind the arguments provided into a dict. When passed a function, a tuple of arguments and a dict of keyword arguments bind_arguments returns a dict of names as the function would see it. This can be useful to implement a cache decorator that uses the function arguments to build the cache key based on the values of the arguments.

  Parameters: | func – the function the arguments should be bound for. args – tuple of positional arguments. kwargs – a dict of keyword arguments.
  Returns: | a `dict` of bound keyword arguments.

## Security Helpers

New in version 0.6.1.

`werkzeug.` `generate_password_hash`(password, method='sha1', salt_length=8)

  Hash a password with the given method and salt with with a string of the given length. The format of the string returned includes the method that was used so that `check_password_hash()` can check the hash.

  The format for the hashed string looks like this:

      method$salt$hash

  This method can not generate unsalted passwords but it is possible to set the method to plain to enforce plaintext passwords. If a salt is used, hmac is used internally to salt the password.

  Parameters: | password – the password to hash method – the hash method to use (`'md5'` or `'sha1'`) salt_length – the lengt of the salt in letters

`werkzeug.` `check_password_hash`(pwhash, password)

  check a password against a given salted and hashed password value. In order to support unsalted legacy passwords this method supports plain text passwords, md5 and sha1 hashes (both salted and unsalted).

  Returns True if the password matched, False otherwise.

  Parameters: | pwhash – a hashed string like returned by `generate_password_hash()` password – the plaintext password to compare against the hash